# Copyright 2011 Tommy Poulter
#
# This file is part of basechip.
#
# basechip is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
# 
# basechip is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with basechip.  If not, see <http://www.gnu.org/licenses/>.

require 'fileutils'
require 'open3'
require 'erb'
# require "bundler/setup"

require 'archive/tar/minitar'
require 'active_support/inflector'
require 'dir'

require 'base_chip/ipc.rb'
require 'base_chip/taskable'
require 'base_chip/task'
require 'base_chip/tasker'
require 'base_chip/task_master'

require 'base_chip/cli'
require 'base_chip/base'
require 'base_chip/dsl'
require 'base_chip/tool'
require 'base_chip/project'
require 'base_chip/generator_menu'
require 'base_chip/list_menu'
require 'base_chip/install_menu'
require 'base_chip/top_menu'
require 'base_chip/git'
module BaseChip
  include Reporting

  @shared_directory = 'foo'

  RANDOM_NAME_PIECES = ('0'..'9').to_a + ('a'..'z').to_a +  ('A'..'Z').to_a
  RANDOM_NAME_PIECES_SIZE = RANDOM_NAME_PIECES.size
  def self.random_string(size=10)
    str = ''
    size.times do
      str += RANDOM_NAME_PIECES[rand(RANDOM_NAME_PIECES_SIZE)]
    end
    str
  end

  GEM_DIR = File.expand_path('../..',  File.realpath(__FILE__))
  def self.root=(root)           ; @root    = root      ; end
  def self.root                  ; @root                ; end
  def self.project=(project)     ; @project = project   ; end

  @@options = OpenStruct.new
  def self.options
    @@options
  end

  def self.find_root(dir = nil, passive=false)
    dir ||= @work_dir ||= Dir.pwd
    @dirs_to_top = []
    loop do
      @dirs_to_top << dir
      break if File.exist? dir + '/base_chip/settings.rb'

      if dir == '/'
        return nil if passive
        fault "Couldn't find file 'base_chip/settings.rb' in any directory above #{Dir.pwd}.\n" +
              "'base_chip/settings.rb' is expected to reside at the top of the checked " +
              "out project, and is generated using the command 'base_chip init'."
      end
      dir = File.expand_path(dir + '/../')
    end
    $project_root = @root = dir
  end

  def self.build_directory_structure(dir, array)
    array.each do |subdir|
      FileUtils.mkdir_p dir + "/" + subdir
    end
  end
  def self.erb_template(in_file, out_file)
    erb = ERB.new(File.read in_file)
    erb.filename = in_file
    File.open(out_file,'w') do |f|
      f.print(erb.result)
    end
  end

  def self.      project_directories ;       @@project_directories end 
  def self.        block_directories ;         @@block_directories end
  def self.configuration_directories ; @@configuration_directories end
  @@common_directories = %w{
      base_chip/tools
      base_chip/recipes
      base_chip/actions
      base_chip/test_lists
      documentation
  } # base_chip/configurations questionable FIXME
  @@configuration_directories = 
    @@common_directories + 
      ( %w{behavioral rtl gates testbench stimulus}.map{|function| %w{verilog classes vhdl systemc headers}.map{|language| "#{function}/#{language}"}}.flatten +
        %w{synthesis timing sw}
      ).map {|function| [function,"top_#{function}"] }.flatten
  @@block_directories = 
    @@configuration_directories +
    %w{
      base_chip/configurations
    }
  @@project_directories = 
    @@common_directories +
    %w{
      base_chip/plugins
      base_chip/subprojects
      base_chip/blocks
      base_chip/configurations
      base_chip/cluster_types
    }

  def self.project(name = nil, &blk)
    if name
      @project = Project.new
      @project.name = name
      @project.blks << blk
    else
      @project
    end
  end
  # SAFE TO DELETE? def self.action(name, &b)
  # SAFE TO DELETE?   @action_builders       ||= {}
  # SAFE TO DELETE?   @action_builders[name] ||= []
  # SAFE TO DELETE?   @action_builders[name] <<  b
  # SAFE TO DELETE? end
  def self.load_environment
    find_root
    file = "#{BaseChip.root}/base_chip/settings.rb"
    instance_eval(File.read(file),file)
    file = '~/.basechip'
    instance_eval(File.read(file),file) if File.exist? file
    @project.configure
  end

  def self.blocks
    @project.blocks.values
  end
  def self.get_children(parents, child_name)
    a = []
    parents.each do |item|
      item.configure
      tmp = item.send(child_name)
      next unless tmp
      a += tmp.values
    end
    a.uniq!
    a
  end
  def self.configurations ; @configurations ||= get_children(self.blocks         , :configurations) end
  def self.actions        ; @actions        ||= get_children(self.configurations , :actions       ) end
  def self.test_lists     ; @test_lists     ||= get_children(self.configurations , :test_lists    ) end
  def self.tests          ; @tests          ||= get_children(self.test_lists     , :tests         ) end
  def self.tools          ; @tools          ||= get_children(self.configurations , :tools         ) end

  def self.version_control
    @version_control ||= (Git.new(root) if File.exist?("#{root}/.git"))
  end

  @@random_generator = nil
  def self.random_generator
    @@random_generator ||= if @@options.seed
                             Random.new(@@options.seed    )
                           else
                             Random.new(rand(0x1_0000_0000))
                           end
  end
  def self.new_random_generator
    if @@options.seed
      random_generator
    else
      Random.new(random_generator.rand(0x1_0000_0000))
    end
  end
end

unless $BASE_CHIP_INCLUDE_MODE
  if BaseChip.find_root(nil,true)
    Dir.glob("#{BaseChip.root}/base_chip/plugins/*").each do |dir|
      file = "#{dir}/lib/#{File.basename(dir)}.rb"
      if File.exist?(file)
        require file
      else
        raise "#{file} doesn't exist within plugin folder #{dir}"
      end
    end
  end
  BaseChip::TopMenu.new.run_cli
end

