# Copyright 2011 Tommy Poulter
#
# This file is part of basechip.
#
# basechip is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
# 
# basechip is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with basechip.  If not, see <http://www.gnu.org/licenses/>.

require 'reporting'
module BaseChip
  module Base
    def self.included mod
      mod.extend Base::ClassMethods
      mod.class_eval do
        include Reporting
        include Base::InstanceMethods

        define_setting :directory, [ String ]

        # chain together projects, blocks, and actions
        attr_reader :project
        attr_reader :block
        attr_reader :configuration
        attr_reader :action
        attr_reader :test_list
        attr_reader :test
        attr_reader :tool
        attr_reader :tool_version
        attr_reader :source_type
        attr_reader :source_language
      end
    end
    module ClassMethods
    end
    module InstanceMethods
      def parent=(parent); @parent = parent; update_chaining; end
      def foster=(parent); @foster = parent; update_chaining; end
      def update_chaining
        # chain blocks, projects, and actions together
        @project         ||= (self.is_a? BaseChip::Project       ) ? self : ((parent.project         if parent) || (foster.project         if foster))
        @block           ||= (self.is_a? BaseChip::Block         ) ? self : ((parent.block           if parent) || (foster.block           if foster))
        @configuration   ||= (self.is_a? BaseChip::Configuration ) ? self : ((parent.configuration   if parent) || (foster.configuration   if foster))
        @action          ||= (self.is_a? BaseChip::Action        ) ? self : ((parent.action          if parent) || (foster.action          if foster))
        @test_list       ||= (self.is_a? BaseChip::TestList      ) ? self : ((parent.test_list       if parent) || (foster.test_list       if foster))
        @test            ||= (self.is_a? BaseChip::Test          ) ? self : ((parent.test            if parent) || (foster.test            if foster))
        @tool            ||= (self.is_a? BaseChip::Tool          ) ? self : ((parent.tool            if parent) || (foster.tool            if foster))
        @tool_version    ||= (self.is_a? BaseChip::ToolVersion   ) ? self : ((parent.tool_version    if parent) || (foster.tool_version    if foster))
        @source_type     ||= (self.is_a? BaseChip::SourceType    ) ? self : ((parent.source_type     if parent) || (foster.source_type     if foster))
        @source_language ||= (self.is_a? BaseChip::SourceLanguage) ? self : ((parent.source_language if parent) || (foster.source_language if foster))
      end                                                                                                                                  

      def configure
        return  if @configured
        super
        # if self.class.children_types.keys.include?(:tool) && self.tools
        #   self.tools.each_value do |t|
        #     t.configure
        #     t.selected_version.configure
        #     t.selected_version.actions.each do |n,a|
        #       self.actions ||= {}
        #       if self.actions[n]
        #         self.actions[n].blks     = a.blks + self.actions[n].blks
        #         self.actions[n].foster ||= a.parent
        #       else
        #         self.actions[n]          = a.clone
        #         self.actions[n].blks     =          self.actions[n].blks.dup
        #         self.actions[n].foster   = a.parent
        #         self.actions[n].parent   = self
        #       end
        #     end
        #   end
        # end
      end

      def discover_configurations
        Dir.glob("#{BaseChip.root}/*/base_chip/configuration.rb").each do |f|
          if f =~ /(\w+)\/base_chip\/configuration\.rb$/
            self.block($1.to_sym) {|b| b.file f}
          end
        end
      end
      def file_glob(glob, regex = nil, type = nil) 
        Dir.glob(glob).each do |f|
          if regex
            if f =~ regex
              self.send(type, $1.to_sym) { |t| t.file(f) }
            else
              fault "fileglob '#{glob}' did not match regex /#{regex}/ in #{self.name}"
            end
          else
            self.file(f) 
          end
        end
      end
    end
  end
end
