# Copyright 2011 Tommy Poulter
#
# This file is part of basechip.
#
# basechip is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
# 
# basechip is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with basechip.  If not, see <http://www.gnu.org/licenses/>.

require 'base_chip/bom_file'

class Hash
  def deep_values
    self.values.map do |v|
      if v.respond_to? :values
        v.values
      else
        v
      end
    end
  end
end

module BaseChip
  class Bom
    attr_reader :hash
    attr_reader :name1
    attr_reader :name2
    def initialize(hash)
      @hash  = hash.with_indifferent_access
      @name1 = 'all'
      @name2 = 'all'
    end
    def verilog   ; @verilog   ||= trim(2,:verilog  ) end
    def classes   ; @classes   ||= trim(2,:classes  ) end
    def vhdl      ; @vhdl      ||= trim(2,:vhdl     ) end
    def systemc   ; @systemc   ||= trim(2,:systemc  ) end
    def headers   ; @headers   ||= trim(2,:headers  ) end

    def rtl       ; @rtl       ||= trim(1,:rtl      ) end
    def gates     ; @gates     ||= trim(1,:gates    ) end
    def testbench ; @testbench ||= trim(1,:testbench) end
    def stimulus  ; @stimulus  ||= trim(1,:stimulus ) end

    def to_a ; @to_a ||= @hash.deep_values.flatten; end
    def list ; @list ||= to_a.join(' ')           ; end
    def to_s ; @to_s ||= list                     ; end

    alias files to_a
    def directories
      @directories ||= files.map{ |f| File.dirname(f) }.uniq
    end

    def +(bom)
      Bom.new(@hash.merge bom.hash)
    end

    protected
    def trim(depth,key)
      hash = HashWithIndifferentAccess.new
      @hash.each do |k1,l1|
        case depth 
        when 1; hash[k1] = l1 if k1.to_s == key.to_s
        when 2; hash[k1] = HashWithIndifferentAccess.new
          l1.each do |k2,l2| 
            hash[k1][k2] = l2 if k2.to_s == key.to_s
          end
        end
      end
      Bom.new(hash)
    end
  end
end
