# Copyright 2011 Tommy Poulter
#
# This file is part of basechip.
#
# basechip is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
# 
# basechip is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with basechip.  If not, see <http://www.gnu.org/licenses/>.

module BaseChip
  class Cluster
    include Dsl
    include Base

    define_settings({:foreground_allowed   => [ Integer               ],
                     :background_allowed   => [ Integer               ],
                     :maximum_jobs         => [ Integer               ],
                     :default_jobs         => [ Integer               ],
                     :id_regex             => [ Regexp                ],
                     :id_variable          => [ String                ],
                     :id_procedure         => [ Proc                  ],
                     :submit_procedure     => [ Proc                  ],
                     :submit_command       => [ String                ],
                     :kill_procedure       => [ Proc                  ],
                     :kill_command         => [ String                ],
                     :cluster_args         => [ String                ]})

    attr_accessor :ids
    attr_accessor :slots
    attr_accessor :slots_used

    def initialize 
      super 
      @ids        = []
      @slots      = 0
      @slots_used = 0
    end

    def available_slots
      @slots - @slots_used
    end

    def submit(command)
      fault "Cluster submission requires a way to kill spawned jobs.  Please specify kill_procedure or kill_command for #{full_name}" unless @kill_procedure || @kill_command

      if submit_procedure
        return submit_procedure.call command
      elsif submit_command
        output = `#{submit_command} #{command}`
        if id_regex
          if result = id_regex.match(output)
            captures = result[1..-1]
            if id = (captures.select{|c| (c!=nil) && (c!='') }.first)
              @ids << id
              return id
            else
              fault "Regex /#{id_regex}/ matched '#{result}' but no useful capture was found in output of submit_command:\n#{output}"
            end
          else
            fault "Regex /#{id_regex}/ could not be matched in output of submit_command:\n#{output}"
          end
        elsif id_procedure
          return id_procedure.call(output)
        else
          fault "job id is required for job tracking.  Please specify id_regex, id_variable or id_procedure for cluster #{full_name}" unless id_variable or id_procedure
        end
      else
        fault "No submit_procedure or submit_command defined for cluster #{full_name}"
      end
    end
    def kill_all
      @ids.each do |id|
        kill id
      end
    end
    def kill(id)
      if kill_procedure
        kill_procedure.call(id)
      elsif kill_command
        system "#{kill_command} #{id}"
      else
        fault "kill_procedure and kill_command for #{full_name} are both missing at time of kill.  This should have been flagged on submit." unless kill_procedure or kill_command
      end
    end
  end
end
