# Copyright 2011 Tommy Poulter
#
# This file is part of basechip.
#
# basechip is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
# 
# basechip is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with basechip.  If not, see <http://www.gnu.org/licenses/>.

require 'base_chip/menu'
module BaseChip
  class GeneratorMenu < Menu
    include Cli
    option :git, :description=>"Check in generated files", :type=>:boolean, :alias=>[:g,:r,:repo]

    desc "Generate a block"
    def block(name)
      BaseChip.load_environment
      BaseChip.find_root(dir = nil)
      @block_name = name
      BaseChip.build_directory_structure(name,BaseChip.block_directories)
      BaseChip.erb_template "#{BaseChip::GEM_DIR}/collateral/block/block.rb.erb", "#{BaseChip.root}/#{name}/base_chip/block.rb"
      FileUtils.cd BaseChip.root
      puts "block '#{name}' generation complete"

      if options.git
        system "git add ./#{name}"
        system "git commit -m 'Initial automated block build of '#{name}' BaseChip.' ./#{name}"

        normal "block '#{name}' has been created."
      end
    end

    desc "Generate a configuration"
    def configuration(block_name, configuration_name)
      BaseChip.load_environment
      BaseChip.find_root(dir = nil)
      @configuration_name = configuration_name
      BaseChip.build_directory_structure("#{block_name}/#{configuration_name}",BaseChip.configuration_directories)
      BaseChip.erb_template "#{BaseChip::GEM_DIR}/collateral/configuration/configuration.rb.erb", "#{BaseChip.root}/#{block_name}/#{configuration_name}/base_chip/configuration.rb"
      FileUtils.cd BaseChip.root
      puts "configuration '#{block_name}:#{configuration_name}' generation complete"

      if options.git
        system "git add ./#{block_name}/#{configuration_name}"
        system "git commit -m 'Initial automated configuration build of '#{configuration_name}' BaseChip.' ./#{block_name}/#{configuration_name}"

        normal "configuration '#{configuration_name}' has been created."
      end
    end

    desc "Generate a tool config file"
    def tool(name)
      puts "tool generation complete"
    end
  end
end
