# Copyright 2011 Tommy Poulter
#
# This file is part of basechip.
#
# basechip is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
# 
# basechip is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with basechip.  If not, see <http://www.gnu.org/licenses/>.

require 'reporting'
module BaseChip
  module Hierarchy
    def self.included mod
      mod.extend Hierarchy::ClassMethods
      mod.class_eval do
        include Reporting
        include Hierarchy::InstanceMethods
      end
    end
    module ClassMethods
    end
    module InstanceMethods
      def configure
        return if @configured
        @directory ||= "#{parent.directory}/#{name}"
        super
      end

      def discover_tools          ; file_glob("#{@directory}/base_chip/tools/*.rb"         , /base_chip\/tools\/(\w+)\.rb$/      , :tool      ) end
      def discover_actions        ; file_glob("#{@directory}/base_chip/actions/*.rb"       , /base_chip\/actions\/(\w+)\.rb$/    , :action    ) end
      def discover_test_lists     ; file_glob("#{@directory}/base_chip/test_lists/*.rb"    , /base_chip\/test_lists\/(\w+)\.rb$/ , :test_list ) end
      def discover_recipes        ; file_glob("#{@directory}/base_chip/recipes/*.rb"                                                          ) end

      def     rtl(       &blk) ; source_type(    :rtl       , &blk) ; end
      def top_rtl(       &blk) ; source_type(:top_rtl       , &blk) ; end
      def     gates(     &blk) ; source_type(    :gates     , &blk) ; end
      def top_gates(     &blk) ; source_type(:top_gates     , &blk) ; end
      def     testbench( &blk) ; source_type(    :testbench , &blk) ; end
      def top_testbench( &blk) ; source_type(:top_testbench , &blk) ; end
      def     stimulus(  &blk) ; source_type(    :stimulus  , &blk) ; end
      def top_stimulus(  &blk) ; source_type(:top_stimulus  , &blk) ; end

      def verilog(       &blk) ; source_language(:verilog   , &blk) ; end
      def classes(       &blk) ; source_language(:classes   , &blk) ; end
      def vhdl(          &blk) ; source_language(:vhdl      , &blk) ; end
      def systemc(       &blk) ; source_language(:systemc   , &blk) ; end
      def headers(       &blk) ; source_language(:headers   , &blk) ; end

      def default_directory_structure_called
        return true if                 @default_directory_structure_called
        return true if parent && parent.default_directory_structure_called
        false
      end
      def default_directory_structure
        return if default_directory_structure_called
                 @default_directory_structure_called = true

            rtl       { |source| source.directories [ "#{source.block.name    }/rtl"       , "#{source.block.name}/#{source.configuration.name    }/rtl"       ] }
        top_rtl       { |source| source.directories [ "#{source.block.name}/top_rtl"       , "#{source.block.name}/#{source.configuration.name}/top_rtl"       ] }
            gates     { |source| source.directories [ "#{source.block.name    }/gates"     , "#{source.block.name}/#{source.configuration.name    }/gates"     ] }
        top_gates     { |source| source.directories [ "#{source.block.name}/top_gates"     , "#{source.block.name}/#{source.configuration.name}/top_gates"     ] }
            testbench { |source| source.directories [ "#{source.block.name    }/testbench" , "#{source.block.name}/#{source.configuration.name    }/testbench" ] }
        top_testbench { |source| source.directories [ "#{source.block.name}/top_testbench" , "#{source.block.name}/#{source.configuration.name}/top_testbench" ] }
            stimulus  { |source| source.directories [ "#{source.block.name    }/stimulus"  , "#{source.block.name}/#{source.configuration.name    }/stimulus"  ] }
        top_stimulus  { |source| source.directories [ "#{source.block.name}/top_stimulus"  , "#{source.block.name}/#{source.configuration.name}/top_stimulus"  ] }

        # FIXME include directory
        verilog { |language| language.append_directories(language.directories.map {|d| "#{d}/#{language.name}"}); language.globs  %w{*.def *.v *.vbh *.sv  } }
        classes { |language| language.       directories(language.directories.map {|d| "#{d}/#{language.name}"}); language.globs  %w{                *.sv  } }
        vhdl    { |language| language.       directories(language.directories.map {|d| "#{d}/#{language.name}"}); language.globs  %w{      *.v *.vhd *.vhdl} }
        systemc { |language| language.       directories(language.directories.map {|d| "#{d}/#{language.name}"}); language.globs  %w{*.c                   } }
        headers { |language| language.       directories(language.directories.map {|d| "#{d}/#{language.name}"}); language.globs  %w{*.*                   } }
      end

    end
  end
end
