# Copyright 2011 Tommy Poulter
#
# This file is part of basechip.
#
# basechip is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
# 
# basechip is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with basechip.  If not, see <http://www.gnu.org/licenses/>.

require 'base_chip/menu'
require 'open-uri'
module BaseChip
  class InstallMenu < Menu
    include Cli
    def before(name)
      BaseChip.load_environment
      # \@table = []
      # case pattern
      # when /^\/(.*)\/$/ ; @pattern = /#\{$1}/
      # else              ; @pattern = pattern
      # end
      # # exit unless BaseChip.project.blocks
    end
    def after
      if @table.size > 0
        print_table(@table,'=>')
      end
    end

    desc "Install recipe by name"
    def recipe(name)
      name_a = name.split /:/
     
      fault "Recipe names should be of the format 'author_name:recipe_name'" unless name_a.size == 2 
      begin
        infile = open("http://basechip.com/recipe_code/#{name_a.first}/#{name_a.last}")
      rescue OpenURI::HTTPError
        fault "Recipe '#{name}' could not be found"
      end
      file = "#{BaseChip.root}/base_chip/recipes/#{name_a.last}.rb"
      out = File.open(file,"w")
      out.puts infile.read
      out.close
      infile.close

      puts "Recipe '#{name}' installed as #{file}"
    end

    desc "Install plugin by name"
    def plugin(name)
      fault "This feature is soon forthcoming"
    end

    desc "Install tool by name"
    def tool(name, version=nil)
      before(name)
      tools = []
      n = name.to_sym
      BaseChip.tools.each do |tool|
        next unless tool.name.to_s == n.to_s
        tools << tool
      end
      unless tools[0]
        puts "Available tools are:"
        ListMenu.new.run_cli(%w{tools --show select_version})
        fault "No tool found named '#{name}'.  Consider installing a recipe to teach basechip how to install '#{name}'"
      end

      versions = []
      tools.each do |tool|
        tool.configure
        if version
          if v = tool.versions[version]
            versions << v
          end
        else
          versions << tool.selected_version
        end
      end
      unless versions[0]
        puts "Available versions for tool '#{name}' are:"
        ListMenu.new.run_cli(['tools', name, '--show', 'available_versions'])
        fault "No version found named '#{version}' for tool '#{name}'."
      end
      puts "Installing version(s) #{versions.map{|v|v.name}.uniq} for tool '#{name}'"
      installed = []
      versions.each do |version|
        next if installed.include? version.name
        installed << version.name
        version.configure
        fault "shared_directory must be specified in project for install locations" unless version.project.shared_directory
        puts   "cd #{version.project.shared_directory}\n#{version.install}"
        system "cd #{version.project.shared_directory}\n#{version.install}"
      end
    end
  end
end
