# Copyright 2011 Tommy Poulter
#
# This file is part of basechip.
#
# basechip is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
# 
# basechip is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with basechip.  If not, see <http://www.gnu.org/licenses/>.

require 'base_chip/menu'
module BaseChip
  class ListMenu < Menu
    include Cli
    def before(pattern)
      BaseChip.load_environment
      @table = []
      case pattern
      when /^\/(.*)\/$/ ; @pattern = /#{$1}/
      else              ; @pattern = pattern
      end
      exit unless BaseChip.project.blocks
    end
    def after
      if @table.size > 0
        print_table(@table,'=>')
      end
    end

    option :show, :description=>"Show a data element for each listed item", :type=>:string, :alias=>[:s]

    desc "List available blocks matching pattern"
    def blocks(pattern=nil)
      before(pattern)
      BaseChip.blocks.each do |block|
        smart_print block.name.to_s, block
      end
      after
    end

    desc "List available configurations matching pattern"
    def configurations(pattern=nil)
      before(pattern)
      BaseChip.configurations.each do |configuration|
        name   = configuration.name
        b_name = configuration.block.name
        smart_print "#{b_name}:#{name}", configuration, b_name.to_s, name.to_s
      end
      after
    end

    desc "List available actions matching pattern"
    def actions(pattern=nil)
      before(pattern)
      BaseChip.actions.each do |action|
        name = action.name
        c = action.configuration
        b = c.block
        smart_print "#{b.name}:#{c.name}:#{name}", action, b.name.to_s, c.name.to_s, name.to_s
      end
      after
    end

    desc "List available test lists matching pattern"
    def test_lists(pattern=nil)
      before(pattern)
      BaseChip.test_lists.each do |list|
        name = list.name
        c = list.configuration
        b = c.block
        smart_print "#{b.name}:#{c.name}:#{name}", list, b.name.to_s, c.name.to_s, name.to_s
      end
      after
    end

    desc "List available tests matching pattern"
    def tests(pattern=nil)
      before(pattern)
      BaseChip.tests.each do |test|
        name = test.name
        l = test.test_list
        c = l.configuration
        b = c.block
        smart_print "#{b.name}:#{c.name}:#{l.name}:#{name}", test, b.name.to_s, c.name.to_s, l.name.to_s, name.to_s
      end
      after
    end

    desc "List available tools matching pattern"
    def tools(pattern=nil)
      before(pattern)
      BaseChip.tools.each do |tool|
        name = tool.name
        smart_print [name,"used by #{tool.configuration.full_name}"], tool, name.to_s
      end
      after
    end

    desc "List available plugins matching pattern"
    option :remote , :description=>"list downloadable plugins", :type=>:string , :alias=>[:r]
    def plugins(pattern=nil)
      fault "This feature is soon forthcoming"
    end

    desc "List available recipes matching pattern"
    option :remote , :description=>"list downloadable recipes", :type=>:string , :alias=>[:r]
    def recipes(pattern=nil)
      begin
        infile = open("http://basechip.com/recipe_list#{"?name=#{pattern}" if pattern}")
      rescue OpenURI::HTTPError
        fault "Could not access basechip.com"
      end
      puts "available recipes are:"
      puts infile.read.gsub(/^/,'  ')
    end

    desc "List the workload that would run if the listed targets were supplied on the command line"
    full_usage "<targets+> [options]"
    default
    def workload(*targets)
      if targets.empty?
        help
        fault "please specify targets"
      end
      BaseChip.load_environment
      @table = []
      BaseChip.project.dereference_workload(targets).each do |item|
        list_line item.full_name, item
      end
      after
    end

  end
end
