# Copyright 2011 Tommy Poulter
#
# This file is part of basechip.
#
# basechip is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
# 
# basechip is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with basechip.  If not, see <http://www.gnu.org/licenses/>.

module BaseChip
  class OutFile
    include Dsl
    include Base
    include TrackState
    define_children({:statistic     =>   Statistic        })
    define_settings({:warning_regex => [ Regexp  , Array ],
                     :error_regex   => [ Regexp  , Array ],
                     :pass_regex    => [ Regexp  , Array ],
                     :optional      => [ Boolean         ],
                     :min_ratio     => [ Array           ],
                     :max_ratio     => [ Array           ],
                     :file_delay    => [ Integer         ]})

    attr_accessor :totals

    def initialize
      super
      @totals = {}
    end
    # def find_parent_values
    #   super 
    #   @warning_regex = to_regex(@warning_regex) if @warning_regex
    #   @error_regex   = to_regex(@error_regex  ) if @error_regex  
    #   @pass_regex    = to_regex(@pass_regex   ) if @pass_regex   
    # end
    def go(name)
      sleep(@file_delay || 0)
      files = Dir.glob(@name.to_s)
      error!(@name,"Could not find any files matching #{@name}") unless files[0] || @optional

      # FIXME fault if file doesn't exist
      files.each do |fname|
        f = File.open(fname,'r')
        f.each do |l| 
          if @warning_regex && (match = @warning_regex.match(l)); warning!(fname,match.to_s) end 
          if @error_regex   && (match = @error_regex  .match(l)); error!(  fname,match.to_s) end 
          if @pass_regex    && (match = @pass_regex   .match(l)); pass!                      end 
          if @statistics
            @statistics.each_value do |s|
              fault("statistic '#{s.full_name}' has no regex defined") unless s.regex
              if match = s.regex.match(l); @totals[s.name] = (match[1] && match[1].to_i) || 1 end
            end
          end
        end
        if @pass_regex && @state.nil?
          error!(fname,"passing banner /#{@pass_regex}/ could not be found")
        end
      end
    end
  end
end
