# Copyright 2011 Tommy Poulter
#
# This file is part of basechip.
#
# basechip is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
# 
# basechip is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with basechip.  If not, see <http://www.gnu.org/licenses/>.

module BaseChip
  module Taskable
    def self.included mod
      mod.extend Taskable::ClassMethods
      mod.class_eval do
        include Taskable::InstanceMethods
        attr_accessor :task       # for pointers to real tasks
        attr_accessor :wait_count
        attr_accessor :next_tasks
        attr_accessor :task_name
        attr_accessor :bundle_name
        attr_accessor :bundle_tgz
        attr_accessor :worker_command
        attr_accessor :foreground
        attr_accessor :totals
        attr_accessor :state
      end
    end
    module ClassMethods
    end
    module InstanceMethods

      def initialize(*args)
        super *args
        initialize_taskable
      end
      def initialize_taskable
        @wait_count = 0
        @next_tasks = []
        @totals     = {}
      end
      def random_name
        BaseChip.random_string
      end

      # nonportable but helpful to be here:
      def deep_depends
        # FIXME circular dependencies
        return @deep_depends if @deep_depends
        return @deep_depends = [] unless self.depends

        @deep_depends = []
        self.depends.each do |depend|
          depend2 = depend.to_s.split(/:/)
          while depend2.size < 3
            depend2.unshift nil
          end
          depend2[0] ||= self.block        .name.to_s
          depend2[1] ||= self.configuration.name.to_s
          @deep_depends += self.project.dereference_workload([depend2.join(':')])
        end
        @deep_depends.each do |d|
          d.deep_configure
          @deep_depends += d.deep_depends
        end
        @deep_depends.uniq!
        @deep_depends
      end
    end
  end
end
