# Copyright 2011 Tommy Poulter
#
# This file is part of basechip.
#
# basechip is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
# 
# basechip is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with basechip.  If not, see <http://www.gnu.org/licenses/>.

require 'base_chip/action'

module BaseChip
  class Test
    include Taskable
    include TrackState
    include Runable
    include Dsl
    include Base
    define_settings({:zip_failing_test   => [ Boolean        ],
                     :zip_passing_test   => [ Boolean        ],
                     :clean_passing_test => [ Boolean        ],
                     :run_test_in_tmp    => [ Boolean        ],
                     :test_actions       => [ Symbol , Array ]}.merge(Action.settings))
    define_children(Action.child_types)

    attr_accessor :original_actions
    attr_accessor :action_extensions
    # def actions
    #   parent.actions
    # end

    def configure
      return if @configured
      name_it
      @directory ||= "#{parent.directory}/#{@name}"
      super
    end
    def deep_configure
      super

      return if @deep_configured
                @deep_configured = true

      @test_actions = [@test_actions] unless @test_actions.is_a? Array
      # @original_actions = @block.dereference_workload([@actions])[0]
      fault "No test_actions exist for test list '#{@test_list}' to run in '#{@configuration.full_name}'" unless @configuration.actions
      unless (@original_actions = @configuration.dereference_workload(@test_actions)) && @original_actions.size > 0
        fault "Test list '#{@test_list.name}' in block:configuration '#{@block.name}:#{@configuration.name}' specified nonexistant action(s) '#{@test_actions}'.  #{@test_list.file}"
      end

      @action_extensions = @original_actions.map do |a|
        append_depends a.depends
        a2 = a.clone
        a2.clear
        a2.parent = self # don't think I need to set foster here, because test list is a peer to action
        a2.deep_configure
        a2
      end

      @action_extensions.sort! do |a1,a2|
        if a1.deep_depends.map{|d|d.name}.include? a2.name
          1
        else
          -1
        end
      end
    end
    # def find_parent_values
    #   super

    #   # @original_actions = @block.dereference_workload([@action])[0]
    #   @original_actions = @block.actions[@action]

    #   @original_actions.         settings.each {|n,s| @children_settings[n] = s if s.value_set and not @children_settings[n].value_set }
    #   @original_actions.children_settings.each {|n,s| @children_settings[n] = s if s.value_set and not @children_settings[n].value_set }

    #   @action_extensions = Action.new
    #   
    #   # FIXME support outfile definition in Test
    #   @original_actions   .out_files.each do |n,f|
    #     @action_extensions.out_files[n] = f.dup
    #   end

    #   @action_extensions.find_parent_values

    #   @action_extensions.name_it
    #   @name = @action_extensions.name
    # end
    # def find_parent_values
    #   super
    #   find_parent_values_of_underlying_action
    # end

    def name_it
      if BaseChip.options.random_bundle_names
        @name ||= random_name
      elsif parent.autoname and not @name
        if parent.autoname.respond_to? :each
          parent.autoname.each do |an|
            if md = /#{an}/.match(@args)
              @name ||= (md[1] || md.to_s)
              break
            end
          end
        else
          if md = /#{parent.autoname}/.match(@args)
            @name ||= (md[1] || md.to_s)
          end
        end
      end
    end
   #def depends(dep=nil)   ;(@action_extensions && @action_extensions.depends            ) || super(dep) end
    def bundle_name        ; name                                                        end
    def bundle_tgz         ; "#{@directory}.tgz"                                         end
    def totals
      return @totals if @totals 
      @totals = {}
      @action_extensions.each do |a|
        @totals.merge! a.totals
      end
    end
    def run
      if @run_test_in_tmp
        bundle_name   = random_string
        run_directory = "/tmp/#{bundle_name}"
      else
        bundle_name   = name
        run_directory = @directory
      end
      FileUtils.rm_rf   run_directory
      FileUtils.mkdir_p run_directory
      Dir.pushd         run_directory

      File.open('seed','w') do |f|
        f.write random_generator.seed
      end
      File.open('rerun.yaml','w') do |f|
        f.write rerun_data.to_yaml
      end

      @action_extensions.each do |a|
        a.random_generator = Random.new(random_generator.rand(0x1_0000_0000))
        a.foreground = @foreground
        a.inner_run
        take_state(a)
        return unless pass?
      end

      Dir.chdir '..'
      if (@clean_passing_test && pass? && !@foreground)
        FileUtils.rm_rf run_directory
      else
        bundle_tgz = nil
        if @zip_failing_test || (@zip_passing_test && pass?)
           bundle_tgz = run_directory + ".tgz"
          @bundle_tgz =    @directory + ".tgz"
          tgz = Zlib::GzipWriter.new(File.open(bundle_tgz, 'wb'))
          Archive::Tar::Minitar.pack(bundle_name, tgz)
          FileUtils.rm_rf bundle_name
        end
        if @run_test_in_tmp
          FileUtils.mkdir_p                           "#{@directory}/../"
          FileUtils.mv((bundle_tgz || run_directory), "#{@directory}/../")
        end
      end
      Dir.popd

      puts "Debug information here: #{@bundle_tgz || @directory}" if @foreground
    end
    # SAFE TO DELETE? def configuration_hash
    # SAFE TO DELETE?   super.merge @action_extensions.configuration_hash
    # SAFE TO DELETE? end

    def clean
    end
    def clobber
      FileUtils.rm_rf directory
      if Dir.glob("#{test_list.directory}/*").empty?
        FileUtils.rm_rf test_list.directory
      end
    end

    # one random generator per test, all subordinate actions use it
    def random_generator
      @random_generator ||= BaseChip.new_random_generator
    end

    def rerun_data
      { 'full_name'         => full_name                          ,
        'seed'              => random_generator.seed              ,
         'append_arguments' => BaseChip.options. append_arguments ,
        'replace_arguments' => BaseChip.options.replace_arguments }
    end
  end
end
