# Copyright 2011 Tommy Poulter
#
# This file is part of basechip.
#
# basechip is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
# 
# basechip is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with basechip.  If not, see <http://www.gnu.org/licenses/>.

require 'base_chip/test'
require 'base_chip/permute'
require 'base_chip/requirement'
require 'base_chip/post'
module BaseChip
  class TestList
    include Dsl
    include Base
    define_settings({:autoname => [ Regexp , Array ]})
    define_children({:test          => Test       ,
                     :permute       => Permute    ,
                     :requirement   => Requirement,
                     :post          => Post       })

    def initialize
      super
      @num_tests = 0
    end
    # def actions
    #   parent.actions
    # end
    alias old_requirement requirement
    alias old_post        post
    def requirement(name=:no_name,&blk); old_requirement(name,&blk) end
    def post(       name=:no_name,&blk); old_post(       name,&blk) end
    def configure
      return if @configured
      @directory ||= "#{configuration.out_directory}/#{name}"
      super
      return unless @tests
      return unless @permutes
      tests = @tests.values
      @permutes.each_value do |permute|
        permute.configure
        next unless permute.permutations
        tests.map! do |t|
          permute.permutations.values.map do |permutation|
            test = t.dup
            test.name = "#{test.name}_#{permute.name}#{permutation.name}".to_sym
            test.blks += permutation.blks
            test
          end
        end
        tests.flatten!
      end

      if @posts
        tests.each do |t|
          @posts.each_value do |p|
            p.blks.each do |b|
              b.parameters.empty? ? t.instance_exec(&b) : b.call(t)
            end
          end
        end
      end
      if @requirements
        new_tests = []
        tests.each do |t|
          t.configure
          ditch = false
          @requirements.each_value do |m|
            m.blks.each do |b|
              ditch = !(b.parameters.empty? ? t.instance_exec(&b) : b.call(t))
              break if ditch
            end
            break if ditch
          end
          next if ditch
          new_tests << t
        end
        tests = new_tests
      end
      @tests = {}
      tests.each do |t|
        name = t.name
        i = 0
        while @tests[name]
          name = "#{t.name}_#{i+=1}"
        end
        t.name = name.to_sym
        @tests[name] = t
      end
    end
    def dereference_workload(tests = nil,passive = false)
      configure
      tests ||= ['all']
      out = []
      return out unless @tests
      tests.each do |t|
        if t == 'all'
          out += @tests.values
        else
          if foo = @tests[t.to_sym]
            out << foo
          else
            fault "Could not find test #{t.inspect} in list #{@name.inspect} in block #{@block.name.inspect}" unless passive
          end
        end
      end
      out
    end

  end
end
