# Copyright 2011 Tommy Poulter
#
# This file is part of basechip.
#
# basechip is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
# 
# basechip is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with basechip.  If not, see <http://www.gnu.org/licenses/>.

require 'base_chip/tool_version'

module BaseChip
  class Tool
    include Dsl
    include Base
    define_settings({:select_version  => [ String    ],
                     :default_version => [ String    ]})
    define_children({:tool_version    =>   ToolVersion})

    def configure
      return if @configured
      super
      @select_version ||= @default_version
      if selected_version
        selected_version.configure
      else
        fault "Selected version '#{@select_version}' for tool '#{@name}' in '#{parent.full_name}' doesn't exist"
      end
    end
    def selected_version
      @selected_version ||= @tool_versions[@select_version] if @tool_versions
    end
    def path(str=:crazy_default_value)
      if str == :crazy_default_value
        @path ||= (selected_version && selected_version.path)
      else
        @path = str
      end
    end
    def available_versions
      tool_versions.keys if tool_versions
    end
  end
end
