# Copyright 2011 Tommy Poulter
#
# This file is part of basechip.
#
# basechip is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
# 
# basechip is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with basechip.  If not, see <http://www.gnu.org/licenses/>.

require 'base_chip/problem'
module BaseChip
  module TrackState
    def self.included mod
      mod.extend TrackState::ClassMethods
      mod.class_eval do
        include TrackState::InstanceMethods
        attr_accessor :state
        attr_accessor :problem
        attr_accessor :problems
      end
    end
    module ClassMethods
    end
    module InstanceMethods
      def pass!        ;          @state ||= 'pass'                                            end
      def warning!(f,s); np(f,s); @state   = 'warning' if @state.nil? or @state.pass?          end
      def error!(  f,s); np(f,s); @state   = (@state =~ /fault/) ? 'error && fault' : 'error'  end
      def fault!(  f,s); np(f,s); @state   = (@state =~ /error/) ? 'error && fault' : 'fault'  end

      def pass?    ; @state == 'pass'    end
      def warning? ; @state == 'warning' end
      def error?   ; @state =~ /fault/   end
      def fault?   ; @state =~ /error/   end

      def np(file,signature)
        return if @problem
        @problem = Problem.new
        @problem.file      = file
        @problem.signature = signature
        @problem
      end

      def take_state(thing)
        return if @problem
        @state   = thing.state
        @problem = thing.problem
      end

      def first_error        ; @problem && @problem.to_s      end
      def first_signature    ; @problem && @problem.signature end
      def first_problem_file ; @problem && @problem.file      end
    end
  end
end
