# Copyright 2011 Tommy Poulter
#
# This file is part of basechip.
#
# basechip is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
# 
# basechip is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with basechip.  If not, see <http://www.gnu.org/licenses/>.

require 'drb'
require 'base_chip/problem'

module BaseChip
  class ReportingError < RuntimeError
    attr :message
    attr :file
    attr :bundle
    def initialize(message,file=nil,bundle=nil)
      if message.is_a? Problem
        @message = message.signature
        @file    = message.file
        @bundle  = message.bundle
      else
        @message = message
        @file    = file
        @bundle  = bundle
      end
    end
  end
  module Reporting
    class Fault < RuntimeError;
      # attr_accessor :msg
      # def initialize(msg)
      #   @msg = msg
      # end
    end
    def self.included mod
      mod.extend Reporting::Methods
      mod.class_eval do
        include Reporting::Methods
      end
    end
    module Methods
      def error(msg)
        re = ReportingError.new(msg)
        message "error", re.message, :red
        raise re
      end
      def fault(msg,exception_msg=nil)
        message "fault", msg, :red # = "#{Process.pid} #{$0} #{ARGV.join( ' ' )} #{msg}"
        DRb.stop_service
        # raise Fault, (exception_msg || msg) # FIXME -- when to raise?
        exit 0
      end
      def normal(msg)
        puts msg unless $quiet
      end
      def verbose(msg)
        puts msg if $verbose
      end
      def debug(msg)
        message "Debug", msg if $debug
      end
      def message(type,msg,color = nil)
        clear_line if self.respond_to? :clear_line
        ostring = case color
                  when :red   ; "[31m"
                  when :green ; "[32m"
                  when nil    ; ''
                  else raise
                  end
        msg.chomp!
        if msg =~ /\n/
          ostring += "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n"
          ostring += "+++++ #{type}: (Multi-line) +++++++++++++++++++++++++++++++++++++++\n"
          ostring += "#{msg}\n"
          ostring += "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n"
        else
          ostring += "#{type}: #{msg}"
        end
        ostring += "[0m" if color
        puts ostring
        ostring
      end
      def class_string
        self.class.to_s.gsub(/BaseChip::/,'')
      end
    end
  end
end
